'use strict';

const math = module.exports = {
    simplifyRectangles: (objects) => {
        for (let i = 0; i < objects.length; ++i) {
            const object = objects[i];
            const polygon = object.polygon;

            // already a rect
            if (!polygon) continue;

            const len = polygon.length;

            // find min/max x/y
            let minX = Number.MAX_VALUE;
            let maxX = -Number.MAX_VALUE;

            let minY = Number.MAX_VALUE;
            let maxY = -Number.MAX_VALUE;

            for (let j = 0; j < len; ++j) {
                const point = polygon[j];

                if (point.x < minX) minX = point.x;
                if (point.x > maxX) maxX = point.x;

                if (point.y < minY) minY = point.y;
                if (point.y > maxY) maxY = point.y;
            }

            // check if each point belongs to the sides of a rectangle
            let allEdges = true;
            let numCorners = 0;

            for (let j = 0; j < len; ++j) {
                const point  = polygon[j];

                if ((point.x === minX || point.x === maxX) && (point.y === minY || point.y === maxY))
                    numCorners++;

                // point is on top or bottom edge
                if (point.x >= minX && point.x <= maxX && (point.y === minY || point.y === maxY))
                    continue;

                // point is on the left or right edge
                if (point.y >= minY && point.y <= maxY && (point.x === minX || point.x === maxX))
                    continue;

                allEdges = false;
            }

            if (allEdges && numCorners === 4) {
                // modify to be rectangle
                delete object.polygon;
                object.x += minX;
                object.y += minY;
                object.width = maxX - minX;
                object.height = maxY - minY;
            }
        }

        return objects;
    },

    combineRectangles: (objects) => {
        for (let i = 0; i < objects.length; ++i) {
            const obj1 = objects[i];

            if (!obj1 || obj1.polygon) continue;

            for (let j = 0; j < objects.length; ++j) {
                const obj2 = objects[j];

                if (i === j) continue;
                if (!obj2 || obj2.polygon) continue;

                // same X and same width and adjacent, combine the Y
                if (obj1.x === obj2.x && obj1.width === obj2.width && obj1.y + obj1.height === obj2.y) {
                    obj1.y = Math.min(obj1.y, obj2.y);
                    obj1.height = obj1.height + obj2.height;
                    objects[j] = null;
                }
                // same Y and same height and adjacent, combine the X
                else if (obj1.y === obj2.y && obj1.height === obj2.height && obj1.x + obj1.width === obj2.x) {
                    obj1.x = Math.min(obj1.x, obj2.x);
                    obj1.width = obj1.width + obj2.width;
                    objects[j] = null;
                }
            }
        }

        return objects.filter((o) => o);
    }
}

function intersects(r1, r2) {
    return !( r2.x > r1.x + r1.width
           || r2.x + r2.width < r1.x
           || r2.y > r1.y + r1.height
           || r2.y + r2.height < r1.y
           );
}

function merge(r1, r2) {
   return { x: Math.min(r1.x, r2.x),
            y: Math.min(r1.y, r2.y),
            w: Math.max(r1.width, r2.width),
            h: Math.max(r1.height, r2.height)
          }
}
